/**
 * 設定
 */

const EXT_NAME = 'GTM Container Switcher';
const RULESET_ID = 279137441;
const error_messages = {
    "invalid_container_id": "適切なコンテナIDを入力してください"
}

const rule_template = {
    "id": RULESET_ID,
    "priority": 1,
    "action": {
        "type": "redirect",
        "redirect": {
            "transform": {
                "queryTransform": {
                    "addOrReplaceParams": [
                        {
                            "key": "id",
                            "value": "GTM-KTCWX6G"
                        }
                    ]
                }
            }
        }
    },
    "condition": {
        "urlFilter": "www.googletagmanager.com/gtm.js",
        "resourceTypes": [
            "script"
        ]
    }
};

/**
 * フィルタが有効なコンテナIDを取得
 * @type {text}
 */

const getActiveFilterContainerID = async () => {
    var rules = await chrome.declarativeNetRequest.getDynamicRules();
    for( var i = 0; i < rules.length; i++){
        if (rules[i].id === RULESET_ID) {
            var params = rules[i]?.action?.redirect?.transform?.queryTransform?.addOrReplaceParams;
            if (!params) return;
            for (var i = 0; i < params.length; i++ ){
                if (params[i].key === "id" && params[i].value.match(/^GTM-[0-9A-Z]+$/)) return (params[i].value);
            }
        }
    }
    return null;
}


/**
 * フィルタリングを開始
 */

const startFiltering = () => {

    // 入力値チェック
    var gtm_container_id = document.getElementById('container-id-input').value;
    if (!gtm_container_id || !gtm_container_id.match(/^GTM-[0-9A-Z]+$/)) {
        alert(error_messages.invalid_container_id)
        return;
    }

    // フィルタを有効化
    var rule = Object.assign({}, rule_template);
    rule.action.redirect.transform.queryTransform.addOrReplaceParams[0].value = gtm_container_id;
    chrome.declarativeNetRequest.updateDynamicRules({ addRules: [rule] });

    // 見た目を変える
    setFilterRunningMode(gtm_container_id)
}

/**
 * フィルタリングが走っている場合
 */

const setFilterRunningMode = (gtm_container_id) =>{
    // 見た目を変更
    document.body.style.backgroundColor = '#ffcccc';
    document.getElementById('status-running').style.display = "flex";
    document.getElementById('status-stopping').style.display = "none";

    // コンテナIDをテキストに表示
    document.getElementById('container-id-text').textContent = gtm_container_id;

    // タイトルを変える
    document.getElementById('ext-title').textContent = `${EXT_NAME} is running`;


}

/**
 * フィルタリングを停止
 */

const endFiltering = () => {

    // フィルタを無効化
    chrome.declarativeNetRequest.updateDynamicRules({ removeRuleIds: [RULESET_ID] })

    // 見た目を変更
    document.body.style.backgroundColor = '#dddddd';
    document.getElementById('status-running').style.display = "none";
    document.getElementById('status-stopping').style.display = "flex";
    document.getElementById('container-id-input').value = '';
    document.getElementById('ext-title').textContent = `${EXT_NAME}`;

}

/**
 * DOM Ready
 */

document.addEventListener('DOMContentLoaded', async () => {

    // ボタンの挙動を設定
    document.getElementById('on').addEventListener('click', async () => {
        startFiltering();
    });
    document.getElementById('off').addEventListener('click', async () => {
        endFiltering();
    });

    // デフォルトの挙動
    var container_id = await getActiveFilterContainerID();
    console.log(container_id)
    if (container_id){
        setFilterRunningMode(container_id);
    }
});
